From cb99544e80606500db711247f266c015d9c32738 Mon Sep 17 00:00:00 2001
From: Tomas Janousek <tomi@nomi.cz>
Date: Wed, 23 May 2007 17:16:06 +0200
Subject: [PATCH] Fix the focus slipping bug on workspace change

Seems like we just have to focus a dummy window if we're going to unmap a
focused window. The focus would slip somewhere otherwise and we ignore the
event in fluxbox.cc (which is right, I believe).

Signed-off-by: Tomas Janousek <tomi@nomi.cz>
---
 src/Screen.cc |   22 ++++++++++++++++++++++
 1 files changed, 22 insertions(+), 0 deletions(-)

diff --git a/src/Screen.cc b/src/Screen.cc
index 4b3092d..9ee5ade 100644
--- a/src/Screen.cc
+++ b/src/Screen.cc
@@ -1202,6 +1202,8 @@ int BScreen::removeLastWorkspace() {
 
 void BScreen::changeWorkspaceID(unsigned int id) {
 
+    bool need_dummy_focus = true;
+
     if (! m_current_workspace || id >= m_workspaces_list.size() ||
         id == m_current_workspace->workspaceID())
         return;
@@ -1224,6 +1226,11 @@ void BScreen::changeWorkspaceID(unsigned int id) {
     for (; it != wins.end(); ++it) {
         if ((*it)->isStuck()) {
             reassociateWindow(*it, id, true);
+
+            // We won't need to focus a dummy window if we don't unmap a stuck
+            // window.
+            if ((*it)->isFocused())
+                need_dummy_focus = false;
         }
     }
 
@@ -1234,6 +1241,17 @@ void BScreen::changeWorkspaceID(unsigned int id) {
             (*icon_it)->setWorkspace(id);
     }
 
+    // We have to create a dummy window here and focus it, because otherwise
+    // the focus would slip to the window under the cursor when unmapping the
+    // focused one.
+    FbTk::FbWindow *dummy_window;
+    if (need_dummy_focus) {
+        dummy_window = new FbTk::FbWindow(screenNumber(),
+                -1, -1, 1, 1, 0, false, false, CopyFromParent, InputOnly);
+        dummy_window->show();
+        dummy_window->setInputFocus(RevertToPointerRoot, CurrentTime);
+    }
+
     currentWorkspace()->hideAll(false);
 
     // set new workspace
@@ -1256,6 +1274,10 @@ void BScreen::changeWorkspaceID(unsigned int id) {
     } else
         FocusControl::revertFocus(*this);
 
+    // The focus should be fine now, destroy the dummy window.
+    if (need_dummy_focus)
+        delete dummy_window;
+
     updateNetizenCurrentWorkspace();
     FbTk::App::instance()->sync(false);
 
-- 
1.5.1.4

